local REGION_NAMES = {
	[1] = 'us',
	[2] = 'kr',
	[3] = 'eu',
	[4] = 'tw',
	[5] = 'cn'
}
local DELIMITER = '::'
local VERSION = '0.0.1';

local transmogSetSlots = {
	INVSLOT_HEAD,
	INVSLOT_SHOULDER,
	INVSLOT_CHEST,
	INVSLOT_WAIST,
	INVSLOT_LEGS,
	INVSLOT_FEET,
	INVSLOT_WRIST,
	INVSLOT_HAND,
	INVSLOT_BACK,
}

local function debugLog(msg)
	print('ArmoryHub debug: ' .. tostring(msg))
end

local function getTransmogSets()
	local sets = {}
	local baseSets =  C_TransmogSets.GetBaseSets()
	for i = 1, #baseSets do
		local info = C_TransmogSets.GetSetInfo(baseSets[i].setID)
		local set = {
			name = info.name,
			label = info.label,
			description = info.description,
			isCollected = info.collected,
			order = info.uiOrder,
			items = {},
		}
		for index, transmogSetSlot in ipairs(transmogSetSlots) do
			local sources = C_TransmogSets.GetSourcesForSlot(baseSets[i].setID, transmogSetSlot)
			for x = 1, #sources do
				local source = sources[x]
				local item = {
					id = source.itemID,
					visualId = source.visualID,
					category = source.categoryID,
					inventory = source.invType,
					setTier = info.description,
					itemMod = source.itemModID,
					isCollected = source.isCollected
				}
				tinsert(set.items, item)
			end
		end
		tinsert(sets, set)
	end
	return sets
end

local function getWardrobe()
	local wardrobe = {}
	for colType = 1, Enum.TransmogCollectionTypeMeta.NumValues - 1 do
		local app = C_TransmogCollection.GetCategoryAppearances(colType)
		for k, o in pairs(app) do
			if o.isCollected and not o.isHideVisual then
				for i, source in ipairs(C_TransmogCollection.GetAppearanceSources(o.visualID)) do
					local item = {}
					local _, _, _, _, _, link = C_TransmogCollection.GetAppearanceSourceInfo(source.sourceID)
					local itemID = GetItemInfoInstant(link)
					item.id = itemID
					item.colType = colType
					tinsert(wardrobe, item)
				end
			end
		end
	end
	return wardrobe
end

local function getCurrency()
	local currency = {}
	local expansion = ''
	for index = 1, C_CurrencyInfo.GetCurrencyListSize() do
		local info = C_CurrencyInfo.GetCurrencyListInfo(index)
		if not info['isHeader'] then
			local item = {}
			item.expansion = expansion
			item.name = info.name
			item.quantity = info.quantity
			tinsert(currency, item)
		else
			expansion = info.name
		end
	end
	return currency
end

SLASH_AHUB1 = '/ahub'
function SlashCmdList.AHUB(msg)
	if msg == 'reset' then
		saveData(true)
	end
end

function saveData(reset)
	local region = REGION_NAMES[GetCurrentRegion()]
	local realm = GetRealmName()
	local character = UnitName('player')
	local wardrobe = getWardrobe()
	local transmogSets = getTransmogSets()
	local hash = region .. ':' .. realm .. ':'.. character
	local currency = getCurrency()
	local isReset = reset or (ArmoryHubExportData and ArmoryHubExportData.version ~= VERSION)

	ArmoryHubExportData = ArmoryHubExportData or {}
	if isReset then
		debugLog('Reset local data.')
		ArmoryHubExportData = {}
	end
	ArmoryHubExportData.version = VERSION
	ArmoryHubExportData[hash] = isReset == false and ArmoryHubExportData[hash] or {}
	ArmoryHubExportData[hash].region = region
	ArmoryHubExportData[hash].realm = realm
	ArmoryHubExportData[hash].character = character
	ArmoryHubExportData[hash].wardrobe = wardrobe
	ArmoryHubExportData[hash].transmogSets = transmogSets
	ArmoryHubExportData[hash].currency = currency

	debugLog('Saved local data.')
end

local frame = CreateFrame('FRAME');
frame:RegisterEvent('PLAYER_LOGIN');
function frame:OnEvent(event, arg1)
	if event == 'PLAYER_LOGIN' then
		saveData(false)
	end
end
frame:SetScript('OnEvent', frame.OnEvent);